#pragma once

#include "log.h"
#include "common.h"

#include <SDL2/SDL.h>
#include <cygnet/Window.h>
#include <cygnet/RenderQueue.h>
#include <optional>

namespace Swan {

class Win {
public:
	Win(Cygnet::Window &window, Cygnet::RenderQueue &queue, float scale):
			window_(window), queue_(queue), scale_(scale) {
	}

	Vec2 getPixSize() {
		return Vec2((float)window_.width() / scale_, (float)window_.height() / scale_);
	}

	Vec2 getSize() {
		return Vec2(
				((float)window_.width() / (scale_ * zoom_)) / TILE_SIZE,
				((float)window_.height() / (scale_ * zoom_)) / TILE_SIZE);
	}

	void onResize(int w, int h) {
		window_.onResize(w, h);
	}

	SDL_Rect createDestRect(Vec2 pos, Vec2 pixsize) {
		return SDL_Rect{
			(int)floor((pos.x - cam_.x) * TILE_SIZE * zoom_),
			(int)floor((pos.y - cam_.y) * TILE_SIZE * zoom_),
			(int)ceil(pixsize.x * zoom_), (int)ceil(pixsize.y * zoom_),
		};
	}

	struct ShowTextureArgs {
		SDL_RendererFlip flip = SDL_FLIP_NONE;
		double hscale = 1;
		double vscale = 1;
		double angle = 0;
		std::optional<SDL_Point> center = std::nullopt;
	};

	void showTexture(
			const Vec2 &pos, SDL_Texture *tex, SDL_Rect *srcrect,
			ShowTextureArgs args) {

		/*
		SDL_Point *center = args.center ? &*args.center : nullptr;
		SDL_Rect destrect = createDestRect(pos, Vec2(srcrect->w * args.hscale, srcrect->h * args.hscale));
		if (SDL_RenderCopyEx(renderer_, tex, srcrect, &destrect, args.angle, center, args.flip) < 0)
			warn << "RenderCopyEx failed: " << SDL_GetError();
			*/
		warn << "Need to implement showTexture";
	}

	// We want an overload which uses RenderCopy instead of RenderCopyEx,
	// because RenderCopy might be faster
	void showTexture(const Vec2 &pos, SDL_Texture *tex, SDL_Rect *srcrect) {
		/*
		SDL_Rect destrect = createDestRect(pos, Vec2(srcrect->w, srcrect->h));
		if (SDL_RenderCopy(renderer_, tex, srcrect, &destrect) < 0)
			warn << "RenderCopy failed: " << SDL_GetError();
			*/
		warn << "Need to implement showTexture";
	}

	void drawRect(const Vec2 &pos, const Vec2 &size) {
		/*
		SDL_Rect destrect = createDestRect(pos, size * TILE_SIZE);
		if (SDL_RenderDrawRect(renderer_, &destrect) < 0)
			warn << "RenderDrawRect failed: " << SDL_GetError();
			*/
		warn << "Need to implement drawRect";
	}

	Vec2 cam_;
	float zoom_ = 1;
	Cygnet::Window &window_;
	Cygnet::RenderQueue &queue_;
	float scale_;
};

}
